//Bibliotheken für den BME680 Sensor
#include <SPI.h>
#include <Adafruit_Sensor.h>
#include "Adafruit_BME680.h"

//Bibliothek für das OLED Display
#include <U8g2lib.h>
#include <Wire.h>

//das OLED Display ist über I2C angeschlossen
#define SDA_PIN 12  //GPIO12 / 6
#define SCL_PIN 14  //GPIO14 / 5

/**
  Konfiguration des Sensors am ESP8266
  Da der verwendete ESP8266 von Ideaspark bereits über ein I2C OLED Display verfügt welches über D5 & D6 
  angeschlossen ist, müssen wir andere Pins für die SPI Kommunikation verwenden!
**/
#define BME_SCK D1
#define BME_MISO D2
#define BME_MOSI D7
#define BME_CS D8

#define SEALEVELPRESSURE_HPA (1013.25)

//erzeugen eines Objektes vom Typ Adafruit_BME680 mit der SPI Konfiguration
Adafruit_BME680 bme(BME_CS, BME_MOSI, BME_MISO, BME_SCK);

//erzeugen einer instanz von U8G2_SSD1306_128X64_NONAME_F_SW_I2C mit der Konfiguration
U8G2_SSD1306_128X64_NONAME_F_SW_I2C u8g2(U8G2_R0, SCL_PIN, SDA_PIN, U8X8_PIN_NONE);

void setup() {
  //starten der seriellen Kommunikation mit 9600 Baud.
  Serial.begin(9600);
  while (!Serial) {
    //Warten bis die serielle Schnittstelle bereit steht.
  }

  //starten der Kommunikation mit dem Sensor.
  //Wenn dieses nicht funktioniert, soll eine Fehlermeldung ausgegeben
  //und eine Endlosschleife gestartet werden.
  if (!bme.begin()) {
    Serial.println("Es konnte kein Sensor vom Typ BME680 gefunden werden!");
    Serial.println("Prüfen Sie die Verkabelung / Konfiguration!");
    while (1)
      ;
  }

  //Setup der Sensordaten (wurde vom Beispiel belassen)
  bme.setTemperatureOversampling(BME680_OS_8X);
  bme.setHumidityOversampling(BME680_OS_2X);
  bme.setPressureOversampling(BME680_OS_4X);
  bme.setIIRFilterSize(BME680_FILTER_SIZE_3);
  bme.setGasHeater(320, 150);  // 320*C for 150 ms

  //beginn der Kommunikation mit dem Display
  u8g2.begin();
  u8g2.clearBuffer();
  //setzen der Schriftgröße auf 8 Punkte
  u8g2.setFont(u8g2_font_ncenB08_tr);
  u8g2.drawStr(0, 26, "BME680 Sensordaten");
  u8g2.setFont(u8g2_font_ncenB08_tr);
  u8g2.drawStr(0, 47, "https://draeger-it.blog");
  u8g2.sendBuffer();
  delay(1500);
}

void displayText(String textLine1, String textLine2) {
  u8g2.clearBuffer();
  //setzen der Schriftgröße auf 10 Punkte
  u8g2.setFont(u8g2_font_ncenB10_tr);
  u8g2.setCursor(10, 12);
  u8g2.print(textLine1);
  //setzen der Schriftgröße auf 14 Punkte
  u8g2.setFont(u8g2_font_ncenB14_tr);
  u8g2.setCursor(10, 40);
  u8g2.print(textLine2);
  u8g2.sendBuffer();
  delay(2500);
}

void loop() {
  //anfordern der aktuellen Sensordaten
  if (!bme.performReading()) {
    Serial.println("Fehler beim lesen der Sensordaten!");
    return;
  }

  //Funktion String(sensorwert, 2) kürzt den Wert auf zwei Stellen nach dem Komma.

  String tempValue = String(bme.temperature, 2);
  String lftdrkValue = String(bme.pressure / 100.0, 2);
  String relLuftfeuchtigkeitValue = String(bme.humidity, 2);
  String hoeheValue = String(bme.readAltitude(SEALEVELPRESSURE_HPA), 2);
  String luftqualitaetValue = String(bme.gas_resistance / 1000.0, 2);

  //auslesen der Temperatur
  String temp = "Temperatur: " + tempValue + " *C";

  //auslesen des Luftdrucks
  String luftdruck = "Luftdruck: " + lftdrkValue + " hPa";

  //auslesen der rel. Luftfeuchtigkeit
  String relLuftfeuchtigkeit = "rel. Luftfeuchtigkeit: " + relLuftfeuchtigkeitValue + "%";

  //auslesen der ungefähren Höhe
  String hoehe = "ungef. Hoehe: " + hoeheValue + " m";

  //berechnen der Luftqualität
  String luftqualitaet = "Luftqualitaet: " + luftqualitaetValue + " kOhm";

  //ein Trenner
  Serial.println("**************************");

  //Ausgeben der ermittelten Daten auf der seriellen Schnittstelle
  Serial.println(temp);
  Serial.println(luftdruck);
  Serial.println(relLuftfeuchtigkeit);
  Serial.println(hoehe);
  Serial.println(luftqualitaet);

  //Ausgeben der Sensordaten auf dem OLED Display
  displayText("Temperatur:", tempValue + " *C");
  displayText("Luftdruck:", lftdrkValue + " hPa");
  displayText("rel. Feuchte:", relLuftfeuchtigkeitValue+ " %");
  displayText("Hoehe:", hoeheValue + " m");
  displayText("Luftqualitaet:", luftqualitaetValue + " kOhm");

  //eine kleine Pause von 2 Sekunden
  delay(2000);
}